@@ -44,6 +44,12 @@ G_DEFINE_TYPE (NMDeviceWifi, nm_device_wifi, NM_TYPE_DEVICE)
 
 void _nm_device_wifi_set_wireless_enabled (NMDeviceWifi *device, gboolean enabled);
 
+typedef struct {
+	NMDeviceWifi *device;
+	NMDeviceWifiRequestScanFn callback;
+	gpointer user_data;
+} RequestScanInfo;
+
 typedef struct {
 	DBusGProxy *proxy;
 
@@ -55,7 +61,8 @@ typedef struct {
 	NMDeviceWifiCapabilities wireless_caps;
 	GPtrArray *aps;
 
-	gboolean wireless_enabled;
+	DBusGProxyCall *scan_call;
+	RequestScanInfo *scan_info;
 } NMDeviceWifiPrivate;
 
 enum {
@@ -66,24 +73,17 @@ enum {
 	PROP_BITRATE,
 	PROP_ACTIVE_ACCESS_POINT,
 	PROP_WIRELESS_CAPABILITIES,
+	PROP_ACCESS_POINTS,
 
 	LAST_PROP
 };
 
-#define DBUS_PROP_HW_ADDRESS "HwAddress"
-#define DBUS_PROP_PERM_HW_ADDRESS "PermHwAddress"
-#define DBUS_PROP_MODE "Mode"
-#define DBUS_PROP_BITRATE "Bitrate"
-#define DBUS_PROP_ACTIVE_ACCESS_POINT "ActiveAccessPoint"
-#define DBUS_PROP_WIRELESS_CAPABILITIES "WirelessCapabilities"
-
 enum {
 	ACCESS_POINT_ADDED,
 	ACCESS_POINT_REMOVED,
 
 	LAST_SIGNAL
 };
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
 /**
@@ -110,7 +110,7 @@ nm_device_wifi_error_quark (void)
  *
  * Creates a new #NMDeviceWifi.
  *
- * Returns: (transfer full): a new WiFi device
+ * Returns: (transfer full): a new Wi-Fi device
  **/
 GObject *
 nm_device_wifi_new (DBusGConnection *connection, const char *path)
@@ -185,9 +185,9 @@ nm_device_wifi_get_mode (NMDeviceWifi *device)
  * nm_device_wifi_get_bitrate:
  * @device: a #NMDeviceWifi
  *
- * Gets the bit rate of the #NMDeviceWifi.
+ * Gets the bit rate of the #NMDeviceWifi in kbit/s.
  *
- * Returns: the bit rate
+ * Returns: the bit rate (kbit/s)
  **/
 guint32
 nm_device_wifi_get_bitrate (NMDeviceWifi *device)
@@ -216,7 +216,7 @@ nm_device_wifi_get_bitrate (NMDeviceWifi *device)
  * nm_device_wifi_get_capabilities:
  * @device: a #NMDeviceWifi
  *
- * Gets the WIFI capabilities of the #NMDeviceWifi.
+ * Gets the Wi-Fi capabilities of the #NMDeviceWifi.
  *
  * Returns: the capabilities
  **/
@@ -271,7 +271,7 @@ nm_device_wifi_get_active_access_point (NMDeviceWifi *device)
  * Gets all the scanned access points of the #NMDeviceWifi.
  *
  * Returns: (element-type NMClient.AccessPoint): a #GPtrArray containing all the
- * scanned #NMAccessPoint<!-- -->s.
+ * scanned #NMAccessPoints.
  * The returned array is owned by the client and should not be modified.
  **/
 const GPtrArray *
@@ -319,28 +319,67 @@ nm_device_wifi_get_access_point_by_path (NMDeviceWifi *device,
 }
 
 static void
-access_point_added (NMObject *self, NMObject *ap)
+request_scan_cb (DBusGProxy *proxy,
+                 DBusGProxyCall *call,
+                 gpointer user_data)
 {
-	g_signal_emit (self, signals[ACCESS_POINT_ADDED], 0, ap);
+	RequestScanInfo *info = user_data;
+	NMDeviceWifiPrivate *priv = NM_DEVICE_WIFI_GET_PRIVATE (info->device);
+	GError *error = NULL;
+
+	dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID);
+
+	if (info->callback)
+		info->callback (info->device, error, info->user_data);
+
+	g_clear_error (&error);
+	g_slice_free (RequestScanInfo, info);
+
+	priv->scan_call = NULL;
+	priv->scan_info = NULL;
 }
 
-static void
-access_point_removed (NMObject *self_obj, NMObject *ap_obj)
+/**
+ * nm_device_wifi_request_scan_simple:
+ * @device: a #NMDeviceWifi
+ * @callback: (scope async) (allow-none): the function to call when the call is done
+ * @user_data: (closure): user data to pass to the callback function
+ *
+ * Request NM to scan for access points on the #NMDeviceWifi. This function only
+ * instructs NM to perform scanning. Use nm_device_wifi_get_access_points()
+ * to get available access points.
+ *
+ * Since: 0.9.8
+ **/
+void
+nm_device_wifi_request_scan_simple (NMDeviceWifi *device,
+                                    NMDeviceWifiRequestScanFn callback,
+                                    gpointer user_data)
 {
-	NMDeviceWifi *self = NM_DEVICE_WIFI (self_obj);
-	NMDeviceWifiPrivate *priv = NM_DEVICE_WIFI_GET_PRIVATE (self);
-	NMAccessPoint *ap = NM_ACCESS_POINT (ap_obj);
+	RequestScanInfo *info;
+	GHashTable *options;
+	NMDeviceWifiPrivate *priv = NM_DEVICE_WIFI_GET_PRIVATE (device);
 
-	if (ap == priv->active_ap) {
-		g_object_unref (priv->active_ap);
-		priv->active_ap = NULL;
-		_nm_object_queue_notify (NM_OBJECT (self), NM_DEVICE_WIFI_ACTIVE_ACCESS_POINT);
+	g_return_if_fail (NM_IS_DEVICE_WIFI (device));
 
-		priv->rate = 0;
-		_nm_object_queue_notify (NM_OBJECT (self), NM_DEVICE_WIFI_BITRATE);
-	}
+	/* If a scan is in progress, just return */
+	if (priv->scan_call)
+		return;
 
-	g_signal_emit (self, signals[ACCESS_POINT_REMOVED], 0, ap);
+	options = g_hash_table_new (g_str_hash, g_str_equal);
+
+	info = g_slice_new0 (RequestScanInfo);
+	info->device = device;
+	info->callback = callback;
+	info->user_data = user_data;
+
+	priv->scan_info = info;
+	priv->scan_call = dbus_g_proxy_begin_call (NM_DEVICE_WIFI_GET_PRIVATE (device)->proxy, "RequestScan",
+	                                           request_scan_cb, info, NULL,
+	                                           DBUS_TYPE_G_MAP_OF_VARIANT, options,
+	                                           G_TYPE_INVALID);
+
+	g_hash_table_unref (options);
 }
 
 static void
@@ -420,8 +459,6 @@ connection_compatible (NMDevice *device, NMConnection *connection, GError **erro
 	NMDeviceWifiCapabilities wifi_caps;
 	const char *key_mgmt;
 
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
 	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
@@ -483,7 +520,19 @@ connection_compatible (NMDevice *device, NMConnection *connection, GError **erro
 		}
 	}
 
-	return TRUE;
+	return NM_DEVICE_CLASS (nm_device_wifi_parent_class)->connection_compatible (device, connection, error);
+}
+
+static GType
+get_setting_type (NMDevice *device)
+{
+	return NM_TYPE_SETTING_WIRELESS;
+}
+
+static const char *
+get_hw_address (NMDevice *device)
+{
+	return nm_device_wifi_get_hw_address (NM_DEVICE_WIFI (device));
 }
 
 /**************************************************************/
@@ -523,6 +572,9 @@ get_property (GObject *object,
 	case PROP_WIRELESS_CAPABILITIES:
 		g_value_set_uint (value, nm_device_wifi_get_capabilities (self));
 		break;
+	case PROP_ACCESS_POINTS:
+		g_value_set_boxed (value, nm_device_wifi_get_access_points (self));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -566,36 +618,38 @@ register_properties (NMDeviceWifi *device)
 		{ NM_DEVICE_WIFI_BITRATE,              &priv->rate },
 		{ NM_DEVICE_WIFI_ACTIVE_ACCESS_POINT,  &priv->active_ap, NULL, NM_TYPE_ACCESS_POINT },
 		{ NM_DEVICE_WIFI_CAPABILITIES,         &priv->wireless_caps },
+		{ NM_DEVICE_WIFI_ACCESS_POINTS,        &priv->aps, NULL, NM_TYPE_ACCESS_POINT, "access-point" },
 		{ NULL },
 	};
 
 	_nm_object_register_properties (NM_OBJECT (device),
 	                                priv->proxy,
 	                                property_info);
+}
 
-	_nm_object_register_pseudo_property (NM_OBJECT (device),
-	                                     priv->proxy,
-	                                     "AccessPoints",
-	                                     &priv->aps,
-	                                     NM_TYPE_ACCESS_POINT,
-	                                     access_point_added,
-	                                     access_point_removed);
+static void
+access_point_removed (NMDeviceWifi *self, NMAccessPoint *ap)
+{
+	NMDeviceWifiPrivate *priv = NM_DEVICE_WIFI_GET_PRIVATE (self);
+
+	if (ap == priv->active_ap) {
+		g_object_unref (priv->active_ap);
+		priv->active_ap = NULL;
+		_nm_object_queue_notify (NM_OBJECT (self), NM_DEVICE_WIFI_ACTIVE_ACCESS_POINT);
+
+		priv->rate = 0;
+		_nm_object_queue_notify (NM_OBJECT (self), NM_DEVICE_WIFI_BITRATE);
+	}
 }
 
 static void
 constructed (GObject *object)
 {
-	NMDeviceWifiPrivate *priv;
+	NMDeviceWifiPrivate *priv = NM_DEVICE_WIFI_GET_PRIVATE (object);
 
 	G_OBJECT_CLASS (nm_device_wifi_parent_class)->constructed (object);
 
-	priv = NM_DEVICE_WIFI_GET_PRIVATE (object);
-
-	priv->proxy = dbus_g_proxy_new_for_name (nm_object_get_connection (NM_OBJECT (object)),
-											NM_DBUS_SERVICE,
-											nm_object_get_path (NM_OBJECT (object)),
-											NM_DBUS_INTERFACE_DEVICE_WIRELESS);
-
+	priv->proxy = _nm_object_new_proxy (NM_OBJECT (object), NULL, NM_DBUS_INTERFACE_DEVICE_WIRELESS);
 	register_properties (NM_DEVICE_WIFI (object));
 
 	g_signal_connect (NM_DEVICE (object),
@@ -608,6 +662,22 @@ static void
 dispose (GObject *object)
 {
 	NMDeviceWifiPrivate *priv = NM_DEVICE_WIFI_GET_PRIVATE (object);
+	GError *error = NULL;
+
+	if (priv->scan_call) {
+		g_set_error_literal (&error, NM_DEVICE_WIFI_ERROR, NM_DEVICE_WIFI_ERROR_UNKNOWN,
+		                     "Wi-Fi device was destroyed");
+		if (priv->scan_info) {
+			if (priv->scan_info->callback)
+				priv->scan_info->callback (NULL, error, priv->scan_info->user_data);
+			g_slice_free (RequestScanInfo, priv->scan_info);
+			priv->scan_info = NULL;
+		}
+		g_clear_error (&error);
+
+		dbus_g_proxy_cancel_call (priv->proxy, priv->scan_call);
+		priv->scan_call = NULL;
+	}
 
 	clean_up_aps (NM_DEVICE_WIFI (object), FALSE);
 	g_clear_object (&priv->proxy);
@@ -640,6 +710,9 @@ nm_device_wifi_class_init (NMDeviceWifiClass *wifi_class)
 	object_class->dispose = dispose;
 	object_class->finalize = finalize;
 	device_class->connection_compatible = connection_compatible;
+	device_class->get_setting_type = get_setting_type;
+	device_class->get_hw_address = get_hw_address;
+	wifi_class->access_point_removed = access_point_removed;
 
 	/* properties */
 
@@ -679,13 +752,13 @@ nm_device_wifi_class_init (NMDeviceWifiClass *wifi_class)
 		 g_param_spec_uint (NM_DEVICE_WIFI_MODE,
 					    "Mode",
 					    "Mode",
-					    NM_802_11_MODE_UNKNOWN, NM_802_11_MODE_INFRA, NM_802_11_MODE_INFRA,
+					    NM_802_11_MODE_UNKNOWN, NM_802_11_MODE_AP, NM_802_11_MODE_INFRA,
 					    G_PARAM_READABLE));
 
 	/**
 	 * NMDeviceWifi:bitrate:
 	 *
-	 * The bit rate of the device.
+	 * The bit rate of the device in kbit/s.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_BITRATE,
@@ -721,14 +794,29 @@ nm_device_wifi_class_init (NMDeviceWifiClass *wifi_class)
 		                    0, G_MAXUINT32, 0,
 		                    G_PARAM_READABLE));
 
+	/**
+	 * NMDeviceWifi:access-points:
+	 *
+	 * List of all Wi-Fi access points the device can see.
+	 *
+	 * Since: 0.9.10
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_ACCESS_POINTS,
+		 g_param_spec_boxed (NM_DEVICE_WIFI_ACCESS_POINTS,
+		                     "AccessPoints",
+		                     "Access Points",
+		                     NM_TYPE_OBJECT_ARRAY,
+		                     G_PARAM_READABLE));
+
 	/* signals */
 
 	/**
 	 * NMDeviceWifi::access-point-added:
-	 * @device: the wifi device that received the signal
+	 * @device: the Wi-Fi device that received the signal
 	 * @ap: the new access point
 	 *
-	 * Notifies that a #NMAccessPoint is added to the wifi device.
+	 * Notifies that a #NMAccessPoint is added to the Wi-Fi device.
 	 **/
 	signals[ACCESS_POINT_ADDED] =
 		g_signal_new ("access-point-added",
@@ -742,10 +830,10 @@ nm_device_wifi_class_init (NMDeviceWifiClass *wifi_class)
 
 	/**
 	 * NMDeviceWifi::access-point-removed:
-	 * @device: the wifi device that received the signal
+	 * @device: the Wi-Fi device that received the signal
 	 * @ap: the removed access point
 	 *
-	 * Notifies that a #NMAccessPoint is removed from the wifi device.
+	 * Notifies that a #NMAccessPoint is removed from the Wi-Fi device.
 	 **/
 	signals[ACCESS_POINT_REMOVED] =
 		g_signal_new ("access-point-removed",
