@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2012 Red Hat, Inc.
+ * (C) Copyright 2007 - 2014 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -68,17 +68,19 @@ typedef enum
 #define NM_SETTING_CONNECTION_ERROR nm_setting_connection_error_quark ()
 GQuark nm_setting_connection_error_quark (void);
 
-#define NM_SETTING_CONNECTION_ID          "id"
-#define NM_SETTING_CONNECTION_UUID        "uuid"
-#define NM_SETTING_CONNECTION_TYPE        "type"
-#define NM_SETTING_CONNECTION_AUTOCONNECT "autoconnect"
-#define NM_SETTING_CONNECTION_TIMESTAMP   "timestamp"
-#define NM_SETTING_CONNECTION_READ_ONLY   "read-only"
-#define NM_SETTING_CONNECTION_PERMISSIONS "permissions"
-#define NM_SETTING_CONNECTION_ZONE        "zone"
-#define NM_SETTING_CONNECTION_MASTER      "master"
-#define NM_SETTING_CONNECTION_SLAVE_TYPE  "slave-type"
-#define NM_SETTING_CONNECTION_SECONDARIES "secondaries"
+#define NM_SETTING_CONNECTION_ID             "id"
+#define NM_SETTING_CONNECTION_UUID           "uuid"
+#define NM_SETTING_CONNECTION_INTERFACE_NAME "interface-name"
+#define NM_SETTING_CONNECTION_TYPE           "type"
+#define NM_SETTING_CONNECTION_AUTOCONNECT    "autoconnect"
+#define NM_SETTING_CONNECTION_TIMESTAMP      "timestamp"
+#define NM_SETTING_CONNECTION_READ_ONLY      "read-only"
+#define NM_SETTING_CONNECTION_PERMISSIONS    "permissions"
+#define NM_SETTING_CONNECTION_ZONE           "zone"
+#define NM_SETTING_CONNECTION_MASTER         "master"
+#define NM_SETTING_CONNECTION_SLAVE_TYPE     "slave-type"
+#define NM_SETTING_CONNECTION_SECONDARIES    "secondaries"
+#define NM_SETTING_CONNECTION_GATEWAY_PING_TIMEOUT "gateway-ping-timeout"
 
 /**
  * NMSettingConnection:
@@ -105,6 +107,8 @@ GType nm_setting_connection_get_type (void);
 NMSetting * nm_setting_connection_new                  (void);
 const char *nm_setting_connection_get_id               (NMSettingConnection *setting);
 const char *nm_setting_connection_get_uuid             (NMSettingConnection *setting);
+NM_AVAILABLE_IN_0_9_10
+const char *nm_setting_connection_get_interface_name   (NMSettingConnection *setting);
 const char *nm_setting_connection_get_connection_type  (NMSettingConnection *setting);
 gboolean    nm_setting_connection_get_autoconnect      (NMSettingConnection *setting);
 guint64     nm_setting_connection_get_timestamp        (NMSettingConnection *setting);
@@ -124,14 +128,26 @@ gboolean    nm_setting_connection_add_permission       (NMSettingConnection *set
                                                         const char *detail);
 void        nm_setting_connection_remove_permission    (NMSettingConnection *setting,
                                                         guint32 idx);
+NM_AVAILABLE_IN_0_9_10
+gboolean    nm_setting_connection_remove_permission_by_value (NMSettingConnection *setting,
+                                                              const char *ptype,
+                                                              const char *pitem,
+                                                              const char *detail);
+
 const char *nm_setting_connection_get_master           (NMSettingConnection *setting);
 gboolean    nm_setting_connection_is_slave_type        (NMSettingConnection *setting,
 							const char *type);
 const char *nm_setting_connection_get_slave_type       (NMSettingConnection *setting);
+
 guint32     nm_setting_connection_get_num_secondaries  (NMSettingConnection *setting);
 const char *nm_setting_connection_get_secondary        (NMSettingConnection *setting, guint32 idx);
 gboolean    nm_setting_connection_add_secondary        (NMSettingConnection *setting, const char *sec_uuid);
 void        nm_setting_connection_remove_secondary     (NMSettingConnection *setting, guint32 idx);
+NM_AVAILABLE_IN_0_9_10
+gboolean    nm_setting_connection_remove_secondary_by_value (NMSettingConnection *setting, const char *sec_uuid);
+
+NM_AVAILABLE_IN_0_9_10
+guint32     nm_setting_connection_get_gateway_ping_timeout (NMSettingConnection *setting);
 
 G_END_DECLS
 
