@@ -179,7 +179,6 @@ verify (NMSetting *setting, GSList *all_settings, GError **error)
 static void
 nm_setting_serial_init (NMSettingSerial *setting)
 {
-	g_object_set (setting, NM_SETTING_NAME, NM_SETTING_SERIAL_SETTING_NAME, NULL);
 }
 
 static void
@@ -256,8 +255,8 @@ nm_setting_serial_class_init (NMSettingSerialClass *setting_class)
 	/**
 	 * NMSettingSerial:baud:
 	 *
-	 * Speed to use for communication over the serial port.  Note that this value
-	 * usually has no effect for mobile broadband modems as they generally
+	 * Speed to use for communication over the serial port.  Note that this
+	 * value usually has no effect for mobile broadband modems as they generally
 	 * ignore speed settings and use the highest available speed.
 	 **/
 	g_object_class_install_property
@@ -269,12 +268,12 @@ nm_setting_serial_class_init (NMSettingSerialClass *setting_class)
 						"broadband modems as they generally ignore speed "
 						"settings and use the highest available speed.",
 						0, G_MAXUINT, 57600,
-						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/**
 	 * NMSettingSerial:bits:
 	 *
-	 * Byte-width of the serial communication.
+	 * Byte-width of the serial communication. The 8 in "8n1" for example.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_BITS,
@@ -283,7 +282,7 @@ nm_setting_serial_class_init (NMSettingSerialClass *setting_class)
 						"Byte-width of the serial communication.  The 8 in "
 						"'8n1' for example.",
 						5, 8, 8,
-						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/**
 	 * NMSettingSerial:parity:
@@ -298,13 +297,13 @@ nm_setting_serial_class_init (NMSettingSerialClass *setting_class)
 						"Parity setting of the serial port.  Either 'E' for even "
 						"parity, 'o' for odd parity, or 'n' for no parity.",
 						'E', 'o', 'n',
-						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/**
 	 * NMSettingSerial:stopbits:
 	 *
 	 * Number of stop bits for communication on the serial port.  Either 1 or 2.
-	 * The 1 in '8n1' for example.
+	 * The 1 in "8n1" for example.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_STOPBITS,
@@ -313,7 +312,7 @@ nm_setting_serial_class_init (NMSettingSerialClass *setting_class)
 						"Number of stop bits for communication on the serial "
 						"port.  Either 1 or 2.  The 1 in '8n1' for example.",
 						1, 2, 1,
-						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/**
 	 * NMSettingSerial:send-delay:
@@ -327,5 +326,5 @@ nm_setting_serial_class_init (NMSettingSerialClass *setting_class)
 						  "Time to delay between each byte sent to the modem, "
 						  "in microseconds.",
 						  0, G_MAXUINT64, 0,
-						  G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
+						  G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
